%% Forecasting with a Panel Tobit Model
%  Laura Liu, Hyungsik Roger Moon, and Frank Schorfheide

%  Empirical Application: Bank Loan Charge-off rates
%  Estimate the model using different methods 
%  NOTE: Please execute main_smpl.R before running main_est.m.

clear all
close all
clc

addpath('../tools');
addpath('estimators');
mkdir temp

data_path = 'data/';
temp_path = 'temp/';
ix_X = 1:2; % []: no covariates
            % 1:2: x_it = [D(ln HPI_{it-1}), D(UR_{it-1})]', baseline setup
            % 1:3: x_it = [D(ln HPI_{it-1}), D(UR_{it-1}), D(ln INC_{it-1})]' 
prior_tuning = 2; % choice of prior tuning parameters (see Section E.1)
                  % 1: initial choice
                  % 2: adjusted choice
spec = ['app_x' strrep(num2str(ix_X),' ','') '_prior' num2str(prior_tuning)];

n_core = feature('numcores'); % number of cores: change if needed

%% Data samples
T = 10;
yr0 = 1994;
qtr0 = 2;
yr1 = 2016;
qtr1 = 1;
all_T = (yr1-yr0)*4+qtr1-qtr0+1;

chargeoff_names = {'ci','cld','mf','cre','hlc','rre','cc','con'};
n_chargeoff = length(chargeoff_names);

load([data_path 'is_idealsmpl.mat'])
v_smpl = find(strcmp(is_idealsmpl,'true'));
n_smpl = length(v_smpl);

v_smpl0 = [493 581]'; % RRE and CC samples starting at 2007Q2 (t=0)
v_smpl1 = [525 602]'; % additional samples: 525: RRE sample starting at 2015Q2 (t=0)
                      %                     602: CC sample starting at 2012Q3 (t=0)

%% Estimation setup
n_param = 3+length(ix_X);
est_names = {'cre_flex_heterosk','cre_normal_heterosk','re_flex_heterosk','re_normal_heterosk', ...
    'cre_flex_homosk','cre_normal_homosk','re_flex_homosk','re_normal_homosk', ...
    'pool_tobit','pool_linear'};
n_est = length(est_names);

B_draw = 1000;
N_draw = 9000;
ix_out = B_draw+(1:N_draw);

% Tuning parameter in normal (mixture) priors 
% [\tau_\theta \tau_\phi \tau_sigma^y]
if prior_tuning == 1 % initial choice
    ttau = [5 5 1]; 
else % adjusted choice
    ttau = [5 20 4];
end

n_sfcst = 7; % order: [Cover. Freq., Ave. Len., Empty Set, {0}, [0,b], {0}U[0,b], Other]
n_dfcst = 2; % order: [LPS, CRPS]

%% Initialize output arrays
Y_out = cell(1,n_smpl);
X_out = cell(1,n_smpl);
sfcstPtwise_stat = cell(n_est,n_smpl);
sfcstAvg_stat = cell(n_est,n_smpl);
dfcst_stat = cell(n_est,n_smpl);
param_est = cell(n_est,n_smpl);
het_est = cell(n_est,n_smpl);

%% Loop over samples
delete(gcp('nocreate'))
parpool('local',n_core);

parfor i_smpl = 1:n_smpl
    %% Load data
    t_smpl = mod(v_smpl(i_smpl)-1,all_T)+1;
    yr = floor((t_smpl+qtr0-2)/4)+yr0;
    qtr = mod(t_smpl+qtr0-2,4)+1;
    i_chargeoff = floor((v_smpl(i_smpl)-1)/all_T)+1;
    smpl = [num2str(yr) 'Q' num2str(qtr) '_' chargeoff_names{i_chargeoff}]
    
    raw = importdata([data_path 't0_' num2str(yr) 'Q' num2str(qtr) '_' ...
        chargeoff_names{i_chargeoff} '.txt']);
    
    id = unique(raw(:,1));
    N = length(id);
    
    Y = nan(T+2,N);
    X = nan(T+2,N,n_param-3);
    
    for t = 1:(T+2)
        for i = 1:N
            ix_it = find((abs(raw(:,1)-id(i))<0.05) ...
                &(abs(raw(:,2)-(yr+(t+qtr-2)/4))<0.05));
            Y(t,i) = raw(ix_it,3);
            if n_param > 3
                X(t,i,:) = raw(ix_it,3+(1:(n_param-3)));
            end
        end
    end
    
    Y_out{i_smpl} = Y;
    X_out{i_smpl} = X;
    
    %% Prepare estimation
    sfcstPtwise_aux = cell(n_est,1);
    sfcstAvg_aux = cell(n_est,1);
    dfcst_aux = cell(n_est,1);
    param_aux = cell(n_est,1);
    het_aux = cell(n_est,1);
    
    %% Tobit estimators  
    for i_est = 1:n_est              
        disp([smpl ', i_est = ' num2str(i_est)])
        
        param0 = cell(2,1);
        if ismember(i_est,1:2:8) % Flexible RE/CRE
            param0{1} = 'flexible'; 
        elseif ismember(i_est,2:2:8)  % Normal RE/CRE
            param0{1} = 'normal';
        end
        param0{2} = ttau;
        
        save_file = cell(1,3);
        if ismember(v_smpl(i_smpl),[v_smpl0 v_smpl1]) && (i_est==1)
            save_file{1} = [temp_path spec '_']; % save MCMC details with provided path and spec
        else
            save_file{1} = []; % not save MCMC details
        end 
        save_file{2} = [smpl '_' est_names{i_est}]; 
            % sample and estimator, for (1) filename and (2) displaying MCMC progress
            
        save_file{3} = ismember(v_smpl(i_smpl),v_smpl0);
            % save_file{1} nonempty: save MCMC details
                % save_file{3} = true: save draws needed for fcst only
                % save_file{3} = false: save draws needed for both fcst and 
                %                       recovering underlying dist
            % save_file{1} empty: not save MCMC details
        
        try
            if ismember(i_est,[1 2]) % Heteroskedastic CRE        
                [sfcstPtwise_aux{i_est},sfcstAvg_aux{i_est},dfcst_aux{i_est},het_aux{i_est}, ...
                    param_aux{i_est}] = tobit_heterosk_cre(Y,X,[B_draw N_draw],param0,save_file);
            elseif ismember(i_est,[3 4]) % Heteroskedastic RE         
                [sfcstPtwise_aux{i_est},sfcstAvg_aux{i_est},dfcst_aux{i_est},het_aux{i_est}, ...
                    param_aux{i_est}] = tobit_heterosk_re(Y,X,[B_draw N_draw],param0,save_file);
            elseif ismember(i_est,[5 6]) % Homoskedastic CRE         
                [sfcstPtwise_aux{i_est},sfcstAvg_aux{i_est},dfcst_aux{i_est},het_aux{i_est}, ...
                    param_aux{i_est}] = tobit_cre(Y,X,[B_draw N_draw],param0,save_file); 
            elseif ismember(i_est,[7 8]) % Homoskedastic RE           
                [sfcstPtwise_aux{i_est},sfcstAvg_aux{i_est},dfcst_aux{i_est},het_aux{i_est}, ...
                    param_aux{i_est}] = tobit_re(Y,X,[B_draw N_draw],param0,save_file);  
            elseif i_est == 9  % Pooled Tobit                
                [sfcstPtwise_aux{i_est},sfcstAvg_aux{i_est},dfcst_aux{i_est}, ...
                    param_aux{i_est}] = pool_tobit(Y,X,[B_draw N_draw],param0,save_file);
            else % Pooled Linear
                [sfcstPtwise_aux{i_est},sfcstAvg_aux{i_est},dfcst_aux{i_est}, ...
                    param_aux{i_est}] = pool_linear(Y,X,[B_draw N_draw],param0,save_file);
            end
        catch
			disp([save_file{2} ' error!'])
            sfcstPtwise_aux{i_est} = nan(1,n_sfcst);
            sfcstAvg_aux{i_est} = nan(1,n_sfcst);
            dfcst_aux{i_est} = nan(1,n_dfcst);
            param_aux{i_est} = nan(1,n_param*3);
            het_aux{i_est} = nan(1+ismember(i_est,1:4),N);
        end     
    end
    
    %% Output
    sfcstPtwise_stat(:,i_smpl) = sfcstPtwise_aux;
    sfcstAvg_stat(:,i_smpl) = sfcstAvg_aux;
    dfcst_stat(:,i_smpl) = dfcst_aux;
    param_est(:,i_smpl) = param_aux;
    het_est(:,i_smpl) = het_aux;    
end

%% Save
delete(gcp('nocreate'))
save([temp_path spec '.mat'])

rmpath('../tools');
rmpath('estimators');