% Generate graphs: Prior and Posterior for CRE Distribution 
% - Scatter plots for moments
% - Historgrams for nubmers of modes
% (for selected samples: v_smpl0 = (RRE and) CC samples starting at 2007Q2)

%% Simulation parameters
rng_seed = 20211001;

thin = 10; % use every 10th MCMC draw, change if needed
NN_draw = N_draw/thin;

x0 = [1 .7085 .7085]; % on the boundary of 50% coverage set (see Section E.1) 

%% Setup style
graph_size_scatter = [6 6];
graph_size_hist = [6 4];
lgd = [];
SFont = 16;

stat_print = {'Mean','SD','Skewness','Kurtosis','Correlation'}; % labels in output graphs
n_stat = length(stat_print);

het_names = {'lambda','y0star'};
n_het = length(het_names);

dist_names = {'pr','post'};
n_dist = length(dist_names);
color_dist_scatter = {'k','b'};
color_dist_hist = [.7 .7 .7; .7 .7 1];

i_est = 1; % Flexible CRE, heterosk.
is_load_est = 1; % load estimation results

%% Simulate draws of mixture distribution parameters from prior
% Hyperparameters
a_alpha0 = 2;
b_alpha0 = 2;
K = 20;

n_coef = length(x0);
n_y = 2;
Phi0 = zeros(n_coef,n_y);
W0 = ttau(2)*eye(n_coef);
Psi0 = 4*diag([ones(1,n_y-1),ttau(3)]);
nu0 = 5+n_y;

% Draws of distribution parameters
rng(rng_seed,'twister');
alpha_draw = gamrnd(a_alpha0,1/b_alpha0,[NN_draw,1]);
zeta_draw = betarnd(ones(NN_draw,K-1),repmat(alpha_draw,1,K-1));
p0_draw = -diff([ones(NN_draw,1) cumprod(1-zeta_draw,2) zeros(NN_draw,1)],1,2);

M0_draw = nan(NN_draw,2,K); % 2nd dim: mean(lambda_i,y*_i0) for each draw and component, given x0
V0_draw = nan(NN_draw,2,2,K); % 2nd and 3rd dims: vcv(lambda_i,y*_i0) for each draw and component
for i_draw = 1:NN_draw
    for k = 1:K
        Sigma_aux = iwishrnd(Psi0,nu0);
        M0_draw(i_draw,:,k) = x0*reshape(mvnrnd(Phi0(:),kron(Sigma_aux,W0)),3,2);
        V0_draw(i_draw,:,:,k) = Sigma_aux;
    end
end

% Grid to recover nubmers of modes
% The grid is based on the prior because the prior is usually more dispersed.
prob_lim = [0.005 0.995]; % covering 0.5th to 99.5th percentiles in each dimension
n_grid = 1000; % number of grids in each dimension

lambday0_gridbase = nan(2,n_grid);
for i_dim = 1:n_y % Note that the marginal dist of (lambda_i,y*_i0) is a t distribution.
    m0 = x0*Phi0(:,i_dim);
    v0 = W0(i_dim,i_dim)*sum(x0.^2);
    a0 = (nu0-1)/2;
    b0 = Psi0(i_dim,i_dim)/2;
    
    aux_xi0 = tinv(prob_lim,2*a0);
    aux_sd = sqrt(b0*v0/a0);
    xxlim = aux_xi0*aux_sd+m0;
    lambday0_gridbase(i_dim,:) = linspace(xxlim(1),xxlim(2),n_grid);
end
[aux1,aux2] = meshgrid(lambday0_gridbase(1,:),lambday0_gridbase(2,:));
lambday0_gridvec = [aux1(:),aux2(:)]; % vectorize the grid, dims = n_grid^2 x 2
    
%% Loop over samples
for i_smpl0 = n_smpl0 % 1:n_smpl0
    %% Prepare: sample i and output
    sub_smpl_i; % subroutine: prepare sample i
    
    stat_draw = nan(NN_draw,n_stat,n_het,n_dist);
    
    %% 1. Prior for (lambda_i,y*_i0)
    i_dist = 1;
    p_draw = p0_draw;
    
    %% 1.1 Scatter plots for moments
    % Reshape the arrays of draws    
    M_draw = permute(M0_draw,[1 3 2]);   % dimensions = NN_draw, K, n_y
    V_draw = permute(V0_draw,[1 4 2 3]); % dimensions = NN_draw, K, n_y, n_y
    
    sub_moments; % subroutine: calculate population moments given mixture parameters
    
    %% 1.2 Historgrams for nubmers of modes
    % Reshape the arrays of draws    
    M_draw = permute(M0_draw,[2 3 1]);   % dimensions = n_y, K, NN_draw
    V_draw = permute(V0_draw,[2 3 4 1]); % dimensions = n_y, n_y, K, NN_draw
    
    sub_nmode; % subroutine: calculate nubmers of modes given mixture parameters
    
    %% 2. Posterior for (lambda_i,y*_i0)
    i_dist = 2;
    
    % Draws of mixture parameters, given x0
    p_draw = est_draw.pp_draw(ix_out(1:thin:N_draw),:,1);
    M1_draw = permute(sum(est_draw.mm_lambday0_draw(ix_out(1:thin:N_draw),:,:,:).*x0,2),[1 3 4 2]);
    V1_draw = est_draw.vv_lambday0_draw(ix_out(1:thin:N_draw),:,:,:);
    
    %% 2.1 Scatter plots for moments
    % Reshape the arrays of draws    
    M_draw = permute(M1_draw,[1 3 2]);   % dimensions = NN_draw, K, n_y
    V_draw = permute(V1_draw,[1 4 2 3]); % dimensions = NN_draw, K, n_y, n_y
    
    sub_moments; % subroutine: calculate population moments given mixture parameters
    
    %% 2.2 Historgrams for nubmers of modes
    % Reshape the arrays of draws
    M_draw = permute(M1_draw,[2 3 1]);   % dimensions = n_y, K, NN_draw
    V_draw = permute(V1_draw,[2 3 4 1]); % dimensions = n_y, n_y, K, NN_draw
    
    sub_nmode; % subroutine: calculate nubmers of modes given mixture parameters
    
    %% 3.1 Plot graphs: scatter plots for moments
    % Set common x,y-scales for the same moments of prior and posterior
    lims = nan(2,n_stat,n_het);
    lims(1,:,:) = nanmin(nanmin(stat_draw,[],1),[],4);
    lims(2,:,:) = nanmax(nanmax(stat_draw,[],1),[],4);
    
    for i_dist = 1:n_dist
        for i_het = n_het % 1:n_het
            for i_stat = 1:n_stat-1
                for j_stat = i_stat+1:n_stat
                    x_var = stat_draw(:,i_stat,i_het,i_dist);
                    y_var = stat_draw(:,j_stat,i_het,i_dist);
                    
                    % Scatter plot
                    F1 = figure;
                    scatter(x_var,y_var,25,color_dist_scatter{i_dist});
                    
                    xlabel(stat_print{i_stat})
                    ylabel(stat_print{j_stat})
                    
                    % Set common x,y-scales
                    xlim(lims(:,i_stat,i_het)')
                    ylim(lims(:,j_stat,i_het)')
                    
                    % Reference lines: mean, skewness, correlation: at 0; kurtosis: at 3
                    if i_stat ~= 2
                        xline(3*(i_stat==4),'k--')
                    end
                    if j_stat ~= 2
                        yline(3*(j_stat==4),'k--')
                    end
                    
                    % Save graph
                    graph_tag = [result_path spec_smpl '_' dist_names{i_dist} '_' ...
                        het_names{i_het} '_' stat_print{i_stat} '_v_' stat_print{j_stat}];                    
                    graph_out(F1,graph_tag,[],SFont,graph_size_scatter,lgd);
                end
            end
            close all
        end
    end
    
    %% 3.2 Plot graphs: historgrams for nubmers of modes
    % Set common x,y-scales for prior and posterior
    min_nmode = min(nmode_draw(:));
    max_nmode = max(nmode_draw(:));
    [counts,centers] = hist(nmode_draw,min_nmode:max_nmode);
    counts_normalized = counts./(sum(counts)*(centers(2)-centers(1)));
    
    xxlim = [min_nmode-0.5 max_nmode+0.5];
    yylim = [0 max(counts_normalized(:))*1.05];
    
    for i_dist = 1:n_dist
        % Histogram
        F1 = figure;
        bar(centers,counts_normalized(:,i_dist),'LineStyle','none', ...
            'FaceColor',color_dist_hist(i_dist,:))
        
        % Set common x,y-scales
        xlim(xxlim)
        ylim(yylim)
        
        % Save graph
        graph_tag = [result_path spec_smpl '_' dist_names{i_dist} '_' het_names{1} het_names{2} ...
            '_nmode'];
        graph_out(F1,graph_tag,[],SFont,graph_size_hist,lgd);
    end
    close all
end